#ifndef INC_MYTESTCLIENT_H
#define INC_MYTESTCLIENT_H
//-----------------------------------------------------------------------------
/*!
\file MyTestClient.h
\brief
\if english
  Header for MyTestClient
\endif

\if german
  Header für MyTestClient
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>
The source code in this file is licensed.

SPDX-License-Identifier:  LicenseRef-Helmut-Jakoby-Commercial-License
                          OR LicenseRef-Helmut-Jakoby-Adapted-Prosperity-Public-License

<b>Helmut-Jakoby-Commercial-License</b><br>
For OEMs (Original Equipment Manufacturers), ISVs (Independent Software Vendors),
VARs (Value Added Resellers) and other distributors who combine commercially licensed
software with <b>Software © by Helmut Jakoby</b> and distribute it as such or in
conjunction with another product and do not wish to publish the source code of the
commercially licensed software under any other license that may be offered, a commercial
license is available for this purpose. Please review the following information to ensure
the Commercial-License requirements will be
met: https://globalobjects.de/hjacomlicense_en.html .

<b>Helmut-Jakoby-Adapted-Prosperity-Public-License</b><br>
For developers of free, non-commercial software applications who want to combine and
distribute their application with <b>Software © by Helmut Jakoby</b>, this license is
a good option. Please review the following information to ensure the
Adapted-Prosperity-Public-License requirements will be
met: https://globalobjects.de/hjappllicense_en.html .

<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include <TcpTypes.h>
#include <TcpClient.h>
//-----------------------------------------------------------------------------
/*!
\brief
\if english
  An example class for a simple client derived from Client.

  Only the method MyTestClient::processServerMessage(tcp::Message * pTcpMessage)
  was overwritten.

  Implementation of MyTestClient:
  <B>MyTestClient.cpp</B>
  \include MyTestClient.cpp

  \anchor TCP_MYTESTCLIENT_USING
  The MyTestClient class can be tried out as follows (the server must be started
  first!):
\endif

\if german
  Eine Beispielklasse für einen einfachen Client abgeleitet von Client.

  Es wurde lediglich die Methode
  MyTestClient::processServerMessage(tcp::Message * pTcpMessage) überschrieben.

  Implementierung von MyTestClient:
  
  <B>MyTestClient.cpp</B>
  \include MyTestClient.cpp

  \anchor TCP_MYTESTCLIENT_USING
  Die Klasse MyTestClient kann folgendermassen ausprobiert werden (es muss vorher
  der Server gestartet sein!):
\endif

<B>ClientMain.cpp</B>
\include ClientMain.cpp

\copyright © 2010 Helmut Jakoby
\author Helmut Jakoby
*/
class MyTestClient : public tcp::Client
{
  protected:
    //============== Konstruktor
    //-------------------------------------------------------------------------
    /*!
    \if english
      This constructor is not available.
    \endif

    \if german
      Dieser Konstruktor steht niccht zur Verfügung.
    \endif
    */
    MyTestClient();
    //-------------------------------------------------------------------------
    /*!
    \if english
      This constructor is not available.
    \endif

    \if german
      Dieser Konstruktor steht niccht zur Verfügung.
    \endif
    */
    MyTestClient( unsigned int,
                  int,
                  int,
                  int,
                  int );
    //-------------------------------------------------------------------------

  public:
    //============== Konstruktoren
    //-------------------------------------------------------------------------
    /*!
    \if english
      The constructor starts the client, but is not yet connected to the server
      from rstrHostname.
      \param[in] rstrHostname The host name or IP address (e.g. "LOCALHOST" or
        "192.168.2.102").
      \param[in] uiPort The port of the server.
      \param[in] iAdressFamily The socket address family to instantiate a socket
        (usually AF_INET ).
      \param[in] iSocketType The socket type (usually SOCK_STREAM ).
      \param[in] iProtocol The socket protocol (usually 0).
      \param[in] iSecTimeOut The time until waiting for a connection or waiting
        for data is aborted.

      WSA is started.

      \exception eut::ErrorNException An exception is thrown if an error occurs.
    \endif

    \if german
      Der Konstruktor startet den Client, ist aber noch nicht mit dem Server aus
      rstrHostname verbunden.
      \param[in] rstrHostname Der Hostname oder IP-Adresse (z.B. "LOCALHOST" oder
        "192.168.2.102").
      \param[in] uiPort Der Port des Servers.
      \param[in] iAdressFamily Die Socket-AdressFamily um einen Socket zu
        instanziieren (i.d.R. AF_INET ).
      \param[in] iSocketType Der Socket-Typ (i.d.R. SOCK_STREAM ).
      \param[in] iProtocol Das Socket-Protokol (i.d.R. 0).
      \param[in] iSecTimeOut Die Zeit bis ein Warten auf eine Verbindung bzw.
         warten auf Daten abgebrochen wird.

      Es wird WSA gestartet.

      \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
    \endif
    */
    MyTestClient( const std::string & rstrHostname,
                  unsigned int uiPort,
                  int iAdressFamily,
                  int iSocketType,
                  int iProtocol,
                  int iSecTimeOut );
    //-------------------------------------------------------------------------
    /*!
    \if english
      The constructor starts the client, but is not yet connected to the server
      from rstrHostname.
      \param[in] rstrHostname The host name or IP address (e.g. "LOCALHOST" or
        "192.168.2.102").
      \param[in] uiPort The port of the server.
      \param[in] iAdressFamily The socket address family to instantiate a socket
        (usually AF_INET ).
      \param[in] iSocketType The socket type (usually SOCK_STREAM ).
      \param[in] iProtocol The socket protocol (usually 0).
      \param[in] uiDataSize The packet size.
      \param[in] iSecTimeOut The time until waiting for a connection or waiting
        for data is aborted.

      WSA is started.

      \exception eut::ErrorNException An exception is thrown if an error occurs.
    \endif

    \if german
      Der Konstruktor startet den Client, ist aber noch nicht mit dem Server aus
      rstrHostname verbunden.
      \param[in] rstrHostname Der Hostname oder IP-Adresse (z.B. "LOCALHOST" oder
        "192.168.2.102").
      \param[in] uiPort Der Port des Servers.
      \param[in] iAdressFamily Die Socket-AdressFamily um einen Socket zu
        instanziieren (i.d.R. AF_INET ).
      \param[in] iSocketType Der Socket-Typ (i.d.R. SOCK_STREAM ).
      \param[in] iProtocol Das Socket-Protokol (i.d.R. 0).
      \param[in] uiDataSize Die Paketgröße.
      \param[in] iSecTimeOut Die Zeit bis ein Warten auf eine Verbindung bzw.
         warten auf Daten abgebrochen wird.

      Es wird WSA gestartet.

      \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
    \endif
    */
    MyTestClient( const std::string & rstrHostname,
                  unsigned int uiPort,
                  int iAdressFamily,
                  int iSocketType,
                  int iProtocol,
                  unsigned int uiDataSize,
                  int iSecTimeOut );
    //-------------------------------------------------------------------------

     //============== Destruktore
    //-------------------------------------------------------------------------
    /*!
    \if english
      In the destructor the connection to the server is terminated if not already
      done.
    \endif

    \if german
      Im Destruktor wird die Verbindung zum Server abgebrochen wenn noch nicht
      geschehen.
    \endif
    */
    virtual ~MyTestClient();
    //-------------------------------------------------------------------------

  private:
    //============== Copy-Konstruktor
    //-------------------------------------------------------------------------
    /*!
    \if english
      The copy constructor is not available.
    \endif

    \if german
      Der Copy-Konstruktor steht nicht zur Verfügung.
    \endif
    */
    MyTestClient( const MyTestClient & );
    //-------------------------------------------------------------------------

  protected:
    //============== Methoden
    //-------------------------------------------------------------------------
    /*!
    \if english
      Outputs a message from the connected server via std::cout.
      \param[in] pTcpMessage The message to be processed from the server.
    \endif

    \if german
      Gibt eine Nachricht vom verbundenen Server über std::cout aus.
      \param[in] pTcpMessage Die zu verarbeitende Nachricht vom Server.
    \endif
    */
    virtual void processServerMessage( tcp::Message * pTcpMessage );
    //-------------------------------------------------------------------------

  private:
    //============== Operatoren
    //-------------------------------------------------------------------------
    /*!
    \if english
      The assignment operator is not available.
    \endif

    \if german
      Der Zuweisungsoperator steht nicht zur Verfügung.
    \endif
    */
    MyTestClient & operator= ( const MyTestClient & );
    //-------------------------------------------------------------------------
};
//-----------------------------------------------------------------------------
#endif
