#ifndef INC_TCPRECEIVER_H
#define INC_TCPRECEIVER_H
//-----------------------------------------------------------------------------
/*!
\file TcpReceiver.h
\brief
\if english
  Header for Receiver.
\endif

\if german
  Header für TcpReceiver.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>
The source code in this file is licensed.

SPDX-License-Identifier:  LicenseRef-Helmut-Jakoby-Commercial-License
                          OR LicenseRef-Helmut-Jakoby-Adapted-Prosperity-Public-License

<b>Helmut-Jakoby-Commercial-License</b><br>
For OEMs (Original Equipment Manufacturers), ISVs (Independent Software Vendors),
VARs (Value Added Resellers) and other distributors who combine commercially licensed
software with <b>Software © by Helmut Jakoby</b> and distribute it as such or in
conjunction with another product and do not wish to publish the source code of the
commercially licensed software under any other license that may be offered, a commercial
license is available for this purpose. Please review the following information to ensure
the Commercial-License requirements will be
met: https://globalobjects.de/hjacomlicense_en.html .

<b>Helmut-Jakoby-Adapted-Prosperity-Public-License</b><br>
For developers of free, non-commercial software applications who want to combine and
distribute their application with <b>Software © by Helmut Jakoby</b>, this license is
a good option. Please review the following information to ensure the
Adapted-Prosperity-Public-License requirements will be
met: https://globalobjects.de/hjappllicense_en.html .

<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"
#include "TcpReceiverTransmitter.h"

#if defined (__PTHREADS)
  #include "EuEndlessPThread.h"
#else
  #include "EuRepeatingThread.h"
#endif

#include "TcpMain.h"
#include <queue>

#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  #include <winsock.h>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  //Forwards
  class Server;
  class AbstractMessage;
  class Client;
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Superclass for all TcpReceiver that communicate via a SOCKET and deliver their
    receive result to their parent object.
  \endif

  \if german
    Oberklasse für alle TcpReceiver, welche über ein SOCKET kommunizieren und
    ihr Empfangsergebnis an ihr Elternobjekt liefert.
  \endif
  \sa #m_pParentTcpMain

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll Receiver : public ReceiverTransmitter
  {
    protected:
      //============== Attribute
      //-----------------------------------------------------------------------
      /*!
      \if english
        The parent object (Client or Server)
      \endif

      \if german
        Das Elternobjekt (Client oder Server).
      \endif
      */
      TcpMain * m_pParentTcpMain;
      //-----------------------------------------------------------------------

      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      Receiver();
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      Receiver( SOCKET Socket );
      //-----------------------------------------------------------------------


    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] pParent The owning parent object.
        \param[in] Socket The socket used for communication.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] pParent Das besitzende Elternobjekt.
        \param[in] Socket Socket über den kommuniziert wird.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      Receiver( TcpMain * pParent, SOCKET Socket );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif

      \if german
        Destruktor.
      \endif
      */
      virtual ~Receiver();
      //-----------------------------------------------------------------------

    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      Receiver( const Receiver & );
      //-----------------------------------------------------------------------

      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      Receiver & operator= ( const Receiver & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------

#endif
