#ifndef INC_TCPSERVERACCEPTRECEIVER_H
#define INC_TCPSERVERACCEPTRECEIVER_H
//-----------------------------------------------------------------------------
/*!
\file TcpServerAcceptReceiver.h
\brief
\if english
  Header for ServerAcceptReceiver.
\endif

\if german
  Header für ServerAcceptReceiver.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>
The source code in this file is licensed.

SPDX-License-Identifier:  LicenseRef-Helmut-Jakoby-Commercial-License
                          OR LicenseRef-Helmut-Jakoby-Adapted-Prosperity-Public-License

<b>Helmut-Jakoby-Commercial-License</b><br>
For OEMs (Original Equipment Manufacturers), ISVs (Independent Software Vendors),
VARs (Value Added Resellers) and other distributors who combine commercially licensed
software with <b>Software © by Helmut Jakoby</b> and distribute it as such or in
conjunction with another product and do not wish to publish the source code of the
commercially licensed software under any other license that may be offered, a commercial
license is available for this purpose. Please review the following information to ensure
the Commercial-License requirements will be
met: https://globalobjects.de/hjacomlicense_en.html .

<b>Helmut-Jakoby-Adapted-Prosperity-Public-License</b><br>
For developers of free, non-commercial software applications who want to combine and
distribute their application with <b>Software © by Helmut Jakoby</b>, this license is
a good option. Please review the following information to ensure the
Adapted-Prosperity-Public-License requirements will be
met: https://globalobjects.de/hjappllicense_en.html .

<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpTypes.h"
#include "TcpReceiver.h"

#include "TcpMain.h"
#include <queue>
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
    #include <winsock.h>
  #endif
  //---------------------------------------------------------------------------
  //Forwards
  class Server;
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    /* 'Bezeichner': Klasse 'Typ' benötigt eine DLL-Schnittstelle, die von... */
    #pragma warning( disable : 4251 )
    /* class 'TYP' ist keine DLL-Schnittstelle und wurde als Basisklasse... */
    #pragma warning( disable : 4275 )
  #endif
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Accept-Receiver accepts new clients and registers them with its parent object
    (usually a Server or a subclass of it).
  \endif

  \if german
    Accept-Receiver nimmt neue Clients an und meldet diese bei seinem Eltern-Objekt
    an (i.d.R. ein Server bzw. eine Unterklasse von diesem).
  \endif

  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  class __tcp_export_dll ServerAcceptReceiver : public Receiver
  {
    protected:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      ServerAcceptReceiver();
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      ServerAcceptReceiver( SOCKET );
      //-----------------------------------------------------------------------
      /*!
      \if english
        This constructor is not available.
      \endif

      \if german
        Dieser Konstruktor steht niccht zur Verfügung.
      \endif
      */
      ServerAcceptReceiver( TcpMain *, SOCKET );
      //-----------------------------------------------------------------------

    public:
      //============== Konstruktoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing.
        \param[in] pParent The owning parent object.
        \param[in] Socket The socket used for communication.
        \exception eut::ErrorNException An exception is thrown if an error occurs.
      \endif

      \if german
        Konstruktor mit Parameterübergabe.
        \param[in] pParent Das besitzende Elternobjekt.
        \param[in] Socket Socket über den kommuniziert wird.
        \exception eut::ErrorNException Bei einem Fehler wird eine Exception geworfen.
      \endif
      */
      ServerAcceptReceiver( Server * pParent, SOCKET Socket );
      //-----------------------------------------------------------------------
      /*!
      \if english
        Destructor.
      \endif

      \if german
        Destruktor.
      \endif
      */
      virtual ~ServerAcceptReceiver();
      //-----------------------------------------------------------------------

    private:
      //============== Copy-Konstruktor
      //-----------------------------------------------------------------------
      /*!
      \if english
        The copy constructor is not available.
      \endif

      \if german
        Der Copy-Konstruktor steht nicht zur Verfügung.
      \endif
      */
      ServerAcceptReceiver( const ServerAcceptReceiver & );
      //-----------------------------------------------------------------------

    protected:
      //========== Methoden ========
      //-----------------------------------------------------------------------
      /*!
      \if english
        Accepts new clients and registers them with its parent object.
      \endif

      \if german
        Nimmt neue Clients an und meldet diese bei seinem Eltern-Objekt an.
      \endif
      */
      virtual void work() override;
      //-----------------------------------------------------------------------
      #if defined (__PTHREADS)
      /*!
      \if english
        When the thread ends, this virtual method is called.
        \note This method only exists when compiled with POSIX threads (defined
          (__PTHREADS)).
      \endif

      \if german
        Wenn der Thread beendet wirde, wird diese virtuale Methode aufgerufen.
        \note Diese Methode gibt es nur, wenn mit POSIX-Threads kompiliert
          (defined (__PTHREADS)).
      \endif
      */
      virtual void cleanUp();
      //-----------------------------------------------------------------------
      #endif

    public:
      //========== Methoden ========
      //-----------------------------------------------------------------------
      /*!
      \if english
        The socket is closed and the thread is terminated.
      \endif

      \if german
        Es wird der Socked geschlossen und der Thread wird beendet.
      \endif
      */
      #if defined (__PTHREADS)
        virtual int cancel();
      #else
        virtual void cancel() override;
      #endif
      //-----------------------------------------------------------------------

    private:
      //============== Operatoren
      //-----------------------------------------------------------------------
      /*!
      \if english
        The assignment operator is not available.
      \endif

      \if german
        Der Zuweisungsoperator steht nicht zur Verfügung.
      \endif
      */
      ServerAcceptReceiver & operator= ( const ServerAcceptReceiver & );
      //-----------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
