#ifndef INC_TCPTYPES_H
#define INC_TCPTYPES_H
//-----------------------------------------------------------------------------
/*!
\file TcpTypes.h
\brief
\if english
  For each library, here 'TcpLib' there is a type file.

  This file will be usually included by every header from the library.
\endif

\if german
  Für jede Bibliothek, hier 'TcpLib' gibt es eine Typen-Datei.

  Diese Datei wird in der Regel von jeder Header aus der Bibliothek eingebunden.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>
The source code in this file is licensed.

SPDX-License-Identifier:  LicenseRef-Helmut-Jakoby-Commercial-License
                          OR LicenseRef-Helmut-Jakoby-Adapted-Prosperity-Public-License

<b>Helmut-Jakoby-Commercial-License</b><br>
For OEMs (Original Equipment Manufacturers), ISVs (Independent Software Vendors),
VARs (Value Added Resellers) and other distributors who combine commercially licensed
software with <b>Software © by Helmut Jakoby</b> and distribute it as such or in
conjunction with another product and do not wish to publish the source code of the
commercially licensed software under any other license that may be offered, a commercial
license is available for this purpose. Please review the following information to ensure
the Commercial-License requirements will be
met: https://globalobjects.de/hjacomlicense_en.html .

<b>Helmut-Jakoby-Adapted-Prosperity-Public-License</b><br>
For developers of free, non-commercial software applications who want to combine and
distribute their application with <b>Software © by Helmut Jakoby</b>, this license is
a good option. Please review the following information to ensure the
Adapted-Prosperity-Public-License requirements will be
met: https://globalobjects.de/hjappllicense_en.html .

<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpErr.h"
#include <string>
//-----------------------------------------------------------------------------
// DLL Define __TCP_EXPORT_DLL
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  #ifdef __TCP_EXPORT_DLL
    #define __tcp_export_dll  __declspec(dllexport)
    #if defined(_MSC_VER)
      #pragma message ( "Benutze Export defines fuer __tcp_export_dll" )
    #endif
  #else
    #define __tcp_export_dll  //__declspec(dllimport)
    #if defined(_MSC_VER)
      #pragma message ( "Benutze Import defines fuer __tcp_export_dll" )
    #endif
  #endif
#elif defined(__GNUC__)
  #ifdef __TCP_EXPORT_DLL
    #define __tcp_export_dll  //__attribute__ ((visibility("default")))
  #else
    #define __tcp_export_dll  //__attribute__ ((visibility ("hidden")))
  #endif
#endif
//=============================================================================
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  /*!
  \if english
    The version, if new, is changed here.

    The file names of the DLL or LIB file are composed of the project name + the
    compiler identification + the version of the library.

    E.g. for Visual Studio 2015 = "$(ProjectName)$(PlatformToolset)101" as target
    name; if the compile target is "TCP", the name becomes "TCPv140101.dll" and
    "TCPv140101.lib".
  \endif

  \if german
    Die Version, wenn neu, wird hier geändert.

    Die Dateibezeichnungen der DLL- bzw. der LIB-Datei setzt sich aus der
    Projektbezeichnung + der Compilerkennung + der Version der Bibliothek
    zusammmen.

    Z.B. für das Visual Studio 2015 = "$(ProjectName)$(PlatformToolset)101" als Zielname;
    wird die Bezeichnung, wenn das Kompilierziel "TCP", zu "TCPv140101.dll"
    und "TCPv140101.lib"
  \endif
  */
  const std::string sTcpVersion = "1.0.3";
  //---------------------------------------------------------------------------
  /*! 
  \if english
    The default TCP packet size.
  \endif

  \if german
    Die voreingestellte TCP-Packetgröße.
  \endif
  */
  #define DEFAULT_DATASIZE 4096
  //---------------------------------------------------------------------------
  /*!
  \if english
    Required for Message between client and server to indicate what is wanted.
  \endif

  \if german
    Wird für Message zwischen Client und Server benötigt, um zu kennzeichnen,
    was gewollt ist.
  \endif
  */
  enum ActionCode
  {
    /*!
    \if english
      Undefined action.
    \endif

    \if german
      Undefinierte Aktion.
    \endif
    */
    ACTION_UNDEF = 0,
    /*!
    \if english
      Action to synchronize the packet size between server and client.
    \endif

    \if german
      Aktion zum synchronisieren der Paketgröße zwischen Server und Client.
    \endif
    */
    ACTION_SYNC_DATA_SIZE = 1,
    /*!
    \if english
      Something is requested from the communication counterpart without waiting for
      a response.
    \endif

    \if german
      Es wird etwas von der Kommunikationsgegenseite angefordert ohne auf eine
      Antwort zu warten.
    \endif
    */
    ACTION_REQUEST = 2,
    /*!
    \if english
      Something is requested from the communication counterpart and a response is
      waited for.
    \endif

    \if german
      Es wird etwas von der Kommunikationsgegenseite angefordert und darauf
      gewartet.
    \endif
    */
    ACTION_REQUEST_WITH_RESPONSE = 3,
    /*!
    \if english
      Something was requested from the communication counterpart without waiting
      for a direct response; this is the response.
    \endif

    \if german
      Es wurde etwas von der Kommunikationsgegenseite angefordert ohne auf eine
      direkte Antwort zu warten; das ist die Antwort.
    \endif
    */
    ACTION_RESPONSE = 4,
    /*!
    \if english
      An info from the communication counterpart.
    \endif

    \if german
      Eine Info von der Kommunikationsgegenseite.
    \endif
    */
    ACTION_INFO = 5
  };
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
#endif
