//-----------------------------------------------------------------------------
/*!
\file MyTestServer.cpp
\brief Source for MyTestServer.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>
The source code in this file is licensed.

SPDX-License-Identifier:  LicenseRef-Helmut-Jakoby-Commercial-License
                          OR LicenseRef-Helmut-Jakoby-Adapted-Prosperity-Public-License

<b>Helmut-Jakoby-Commercial-License</b><br>
For OEMs (Original Equipment Manufacturers), ISVs (Independent Software Vendors),
VARs (Value Added Resellers) and other distributors who combine commercially licensed
software with <b>Software © by Helmut Jakoby</b> and distribute it as such or in
conjunction with another product and do not wish to publish the source code of the
commercially licensed software under any other license that may be offered, a commercial
license is available for this purpose. Please review the following information to ensure
the Commercial-License requirements will be
met: https://globalobjects.de/hjacomlicense_en.html .

<b>Helmut-Jakoby-Adapted-Prosperity-Public-License</b><br>
For developers of free, non-commercial software applications who want to combine and
distribute their application with <b>Software © by Helmut Jakoby</b>, this license is
a good option. Please review the following information to ensure the
Adapted-Prosperity-Public-License requirements will be
met: https://globalobjects.de/hjappllicense_en.html .

<!-- @@TermsOfLicense END -->
Laber
*/
//-----------------------------------------------------------------------------
#include "MyTestServer.h"
#include <iostream>
//-----------------------------------------------------------------------------
#define SUPER Server
//-----------------------------------------------------------------------------
MyTestServer::MyTestServer()
: SUPER()
{
}
//-----------------------------------------------------------------------------
MyTestServer::MyTestServer( unsigned int uiPort,
                            int iAdressFamily,
                            int iSocketType,
                            int iProtocol,
                            int iSecTimeOut )
: SUPER( uiPort, iAdressFamily, iSocketType, iProtocol, iSecTimeOut )
{
}
//-----------------------------------------------------------------------------
MyTestServer::MyTestServer( unsigned int uiPort,
                            int iAdressFamily,
                            int iSocketType,
                            int iProtocol,
                            unsigned int uiDataSize,
                            int iSecTimeOut )
: SUPER( uiPort, iAdressFamily, iSocketType, iProtocol, uiDataSize, iSecTimeOut )
{
}
//-----------------------------------------------------------------------------
MyTestServer::MyTestServer( unsigned int uiPort,
                            int iAdressFamily,
                            int iSocketType,
                            int iProtocol,
                            unsigned int uiDataSize,
                            unsigned long ulMaxClients,
                            int iSecTimeOut )
: SUPER( uiPort, iAdressFamily, iSocketType, iProtocol, uiDataSize, ulMaxClients, iSecTimeOut )
{
}
//-----------------------------------------------------------------------------
MyTestServer::~MyTestServer()
{
}
//-----------------------------------------------------------------------------
void MyTestServer::processClientMessage( tcp::AbstractMessage * pTcpMessage )
{
  tcp::Message             * t_pTcpMessage = dynamic_cast<tcp::Message*>( pTcpMessage );
  tcp::ServerMessage       * t_pTcpServerMessage = dynamic_cast<tcp::ServerMessage*>( pTcpMessage );
  tcp::ServerAcceptMessage * t_pTcpServerAcceptMessage = dynamic_cast<tcp::ServerAcceptMessage*>( pTcpMessage );
  
  if ( t_pTcpServerMessage )
  {
    if ( t_pTcpServerMessage->getActionCode() < 0 )
    {
      std::cout << "server get error-msg from client[" <<  t_pTcpServerMessage->getSocket() <<"] with code [" << t_pTcpServerMessage->getActionCode() << "]" << std::endl;
    }
    else
    {
      std::cout << "server get client-msg from client[" << t_pTcpServerMessage->getSocket() << "] with code [" << t_pTcpServerMessage->getActionCode() << "]";
      if ( t_pTcpServerMessage->getRawMsgSize() )
      {
        std::cout << " - " << t_pTcpServerMessage->getRawMsgPt();
      }
      std::cout << std::endl;
    }
  }
  else if ( t_pTcpServerAcceptMessage )
  {
    std::cout << "new client:" << t_pTcpServerAcceptMessage->getSocket() << std::endl;
  }
  else if ( t_pTcpMessage )
  {
    std::cout << "Err! No message!" << std::endl;
  }
  if ( pTcpMessage ) SUPER::processClientMessage( pTcpMessage );
}
//-----------------------------------------------------------------------------
int MyTestServer::sendToAllClients( std::string strMsg )
{
  int                                                         t_iErr = 0;
  std::map< SOCKET, ServerReceiverTransmitter * >::iterator   m_ClientMapIterator;
  char                                                      * t_pchBuffer = 0;
  tcp::Message                                              * t_pTcpMessage = 0;
  
  if ( m_ClientMap.size() > 0 )
  {
    for ( m_ClientMapIterator = m_ClientMap.begin();
          m_ClientMapIterator != m_ClientMap.end();
          m_ClientMapIterator++ )
    {
      t_pchBuffer = new char[ strlen( strMsg.c_str() ) + 1 ];
      strcpy(t_pchBuffer, strMsg.c_str());
      t_pchBuffer[ strlen( strMsg.c_str() ) ] = '\x00';
      t_pTcpMessage = new tcp::Message( t_pchBuffer, strlen( strMsg.c_str() ) + 1, 0, 0, "" );
      // int acceptOrder( Message * pTcpMessage  );
      t_iErr = (*m_ClientMapIterator).second->send( t_pTcpMessage );
    }
  }
  return t_iErr;
}
//-----------------------------------------------------------------------------
void MyTestServer::insertNewClient( Server::ServerReceiverTransmitter * pNewServerReceiverTransmitter )
{
  int              t_iErr(0);
  std::string      t_sIPAdress;
  unsigned int    t_uiPort(0);
  
  SUPER::insertNewClient( pNewServerReceiverTransmitter );
  t_iErr = pNewServerReceiverTransmitter->getPeerName( t_sIPAdress, t_uiPort );
  std::cout << "new client ip:" << t_sIPAdress << " port:" << t_uiPort << " err:" << t_iErr << std::endl;

}
//-----------------------------------------------------------------------------
#undef SUPER
//-----------------------------------------------------------------------------
