//-----------------------------------------------------------------------------
/*!
\file TcpReceiver.cpp
\brief Source for Receiver.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>
The source code in this file is licensed.

SPDX-License-Identifier:  LicenseRef-Helmut-Jakoby-Commercial-License
                          OR LicenseRef-Helmut-Jakoby-Adapted-Prosperity-Public-License

<b>Helmut-Jakoby-Commercial-License</b><br>
For OEMs (Original Equipment Manufacturers), ISVs (Independent Software Vendors),
VARs (Value Added Resellers) and other distributors who combine commercially licensed
software with <b>Software © by Helmut Jakoby</b> and distribute it as such or in
conjunction with another product and do not wish to publish the source code of the
commercially licensed software under any other license that may be offered, a commercial
license is available for this purpose. Please review the following information to ensure
the Commercial-License requirements will be
met: https://globalobjects.de/hjacomlicense_en.html .

<b>Helmut-Jakoby-Adapted-Prosperity-Public-License</b><br>
For developers of free, non-commercial software applications who want to combine and
distribute their application with <b>Software © by Helmut Jakoby</b>, this license is
a good option. Please review the following information to ensure the
Adapted-Prosperity-Public-License requirements will be
met: https://globalobjects.de/hjappllicense_en.html .

<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpReceiver.h"
#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <stdlib.h>
  #include <unistd.h>
  #define WSAENOTSOCK 10038L
  #define WSAECONNRESET 10054L
#endif
#ifdef _DEBUG
  #include <iostream>
  #include <sstream>
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #define SUPER ReceiverTransmitter
  //---------------------------------------------------------------------------
  Receiver::Receiver()
  : SUPER(),
    m_pParentTcpMain( nullptr )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "tcp::Receiver";
    #endif
  }
  //---------------------------------------------------------------------------
  Receiver::Receiver( SOCKET Socket )
  : SUPER( Socket ),
    m_pParentTcpMain( nullptr )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "tcp::Receiver";
    #endif
  }
  //---------------------------------------------------------------------------
  Receiver::Receiver( TcpMain * pParent, SOCKET Socket )
  : SUPER( Socket ),
    m_pParentTcpMain( pParent )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "tcp::Receiver";
    #endif
  }
  //---------------------------------------------------------------------------
  Receiver::~Receiver()
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class ReceiverTransmitter' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  Receiver::Receiver( const Receiver & )
  : m_pParentTcpMain( nullptr )
  {
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class ReceiverTransmitter' should be explicitly initialized in the copy constructor
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  Receiver & Receiver::operator= ( const Receiver & )
  {
      return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
