//-----------------------------------------------------------------------------
/*!
\file TcpServerAcceptReceiver.cpp
\brief ServerAcceptReceiver.
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>
The source code in this file is licensed.

SPDX-License-Identifier:  LicenseRef-Helmut-Jakoby-Commercial-License
                          OR LicenseRef-Helmut-Jakoby-Adapted-Prosperity-Public-License

<b>Helmut-Jakoby-Commercial-License</b><br>
For OEMs (Original Equipment Manufacturers), ISVs (Independent Software Vendors),
VARs (Value Added Resellers) and other distributors who combine commercially licensed
software with <b>Software © by Helmut Jakoby</b> and distribute it as such or in
conjunction with another product and do not wish to publish the source code of the
commercially licensed software under any other license that may be offered, a commercial
license is available for this purpose. Please review the following information to ensure
the Commercial-License requirements will be
met: https://globalobjects.de/hjacomlicense_en.html .

<b>Helmut-Jakoby-Adapted-Prosperity-Public-License</b><br>
For developers of free, non-commercial software applications who want to combine and
distribute their application with <b>Software © by Helmut Jakoby</b>, this license is
a good option. Please review the following information to ensure the
Adapted-Prosperity-Public-License requirements will be
met: https://globalobjects.de/hjappllicense_en.html .

<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "TcpServerAcceptReceiver.h"
#include "TcpServer.h"
#if defined (__GNUC__) && ! defined (__MINGW32__)
  #include <stdlib.h>
  #include <unistd.h>
  #define WSAENOTSOCK 10038L
  #define WSAECONNRESET 10054L
#endif
#ifdef _DEBUG
  #include <iostream>
  #include <sstream>
#endif
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
#if defined(_MSC_VER) || defined(__MINGW32__) || defined(__MINGW64__)
  /*
   * WinSock 2 extension -- manifest constants for shutdown()
   */
  #define SD_RECEIVE      0x00
  #define SD_SEND         0x01
  #define SD_BOTH         0x02
#endif
//-----------------------------------------------------------------------------
namespace tcp
{
  //---------------------------------------------------------------------------
  #define SUPER Receiver
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::ServerAcceptReceiver()
  : SUPER()
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerAcceptReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::ServerAcceptReceiver( SOCKET Socket )
  : SUPER( Socket )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerAcceptReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::ServerAcceptReceiver( TcpMain * pParent, SOCKET Socket )
  : SUPER( pParent, Socket )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerAcceptReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::ServerAcceptReceiver( Server * pParent, SOCKET Socket )
  : SUPER( pParent, Socket )
  {
    #ifdef _DEBUG
      m_strDebugInfo = "ServerAcceptReceiver";
    #endif
  }
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::~ServerAcceptReceiver()
  {
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: base class 'class Receiver' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  ServerAcceptReceiver::ServerAcceptReceiver( const ServerAcceptReceiver & )
  {
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: base class 'class Receiver' should be explicitly initialized in the copy constructor [-Wextra]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: comparison between signed and unsigned integer expressions [-Wsign-compare]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsign-compare"
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wsign-conversion"
  #endif
  //---------------------------------------------------------------------------
  void ServerAcceptReceiver::work()
  {
    SOCKET    t_NewConnectSocket = 0;

    #if defined (__PTHREADS)
      m_eThreadStatus = EuPTHREAD_WORKING;
    #endif
    // Blockiert!
    t_NewConnectSocket = accept( m_Socket, nullptr, nullptr );
    #if defined (__PTHREADS)
      pthread_testcancel();
    #else
      if ( this->testCancel() )
      {
        return;
      }
    #endif
    // wenn m_Socket noch da...
    if ( m_Socket )
    {
      int               t_iErr = 0;
      AbstractMessage * t_pTcpAbstractMessage = nullptr;
      Server          * t_pMyParent = dynamic_cast<Server*>( m_pParentTcpMain );
      
      if( t_NewConnectSocket == SOCKET_ERROR)
      {
        std::string   t_strErrMsg( "AcceptErr" );
        #if defined(_MSC_VER) || defined(__MINGW32__)
          int         t_iActionCode =  WSAGetLastError();
        #elif defined(__GNUC__)
          int         t_iActionCode = errno;
        #endif

        t_pTcpAbstractMessage = new Message(  nullptr,       // RawMsg
                                              0,              // RawMsgSize
                                              0,              // MsgID
                                              t_iActionCode,  // ActionCode
                                              t_strErrMsg );  // Info
        #ifdef _DEBUG_TcpMessage
          t_pTcpAbstractMessage->m_sDebugInstanceInfo = "ServerAcceptReceiver::work()";
        #endif
        // wird im Parent wieder gelöscht!
        t_iErr = t_pMyParent->acceptClientMessage( t_pTcpAbstractMessage );
      }
      else
      {
        t_pTcpAbstractMessage = new ServerAcceptMessage( t_NewConnectSocket );
        #ifdef _DEBUG_TcpMessage
          t_pTcpAbstractMessage->m_sDebugInstanceInfo = "ServerAcceptReceiver::work()";
        #endif
        // wird im Parent wieder gelöscht!
        t_iErr = t_pMyParent->acceptClientMessage( t_pTcpAbstractMessage );
      }
      if ( t_iErr )
      {
        delete t_pTcpAbstractMessage;
      }
    }
  }
  //---------------------------------------------------------------------------
  #if defined (__GNUC__)
    // Warnung: comparison between signed and unsigned integer expressions [-Wsign-compare]
    #pragma GCC diagnostic pop
    // Warnung: implicit conversion changes signedness: A to B [-Wsign-conversion]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #if defined (__PTHREADS)
  void ServerAcceptReceiver::cleanUp()
  {
    SUPER::cleanUp();
  }
  #endif
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wshorten-64-to-32"
  #endif
  //---------------------------------------------------------------------------
  #ifdef __PTHREADS
    int ServerAcceptReceiver::cancel()
  #else
    void ServerAcceptReceiver::cancel()
  #endif
  {
    if ( m_Socket )
    {
      #if defined(_MSC_VER) || defined(__MINGW32__)
        shutdown( m_Socket, SD_BOTH );
        closesocket( m_Socket );
      #elif defined(__GNUC__)
        shutdown( m_Socket, SHUT_RDWR );
        close( m_Socket );
      #endif
      m_Socket = 0;
    }
    #ifdef __PTHREADS
      return SUPER::cancel();
    #else
      SUPER::cancel();
    #endif
  }
  //---------------------------------------------------------------------------
  #if defined (__clang__)
    // Warnung: implicit conversion loses integer precision: A to B [-Wshorten-64-to-32]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  ServerAcceptReceiver & ServerAcceptReceiver::operator= ( const ServerAcceptReceiver & )
  {
    return *this;
  }
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace tcp
//-----------------------------------------------------------------------------
