//-----------------------------------------------------------------------------
/*!
\file EuCriticalSection.cpp
\brief Source for CriticalSection
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>
The source code in this file is licensed.

SPDX-License-Identifier:  LicenseRef-Helmut-Jakoby-Commercial-License
                          OR LicenseRef-Helmut-Jakoby-Adapted-Prosperity-Public-License
                          OR AGPL-3.0-or-later

<b>Helmut-Jakoby-Commercial-License</b><br>
For OEMs (Original Equipment Manufacturers), ISVs (Independent Software Vendors),
VARs (Value Added Resellers) and other distributors who combine commercially licensed
software with <b>Software © by Helmut Jakoby</b> and distribute it as such or in
conjunction with another product and do not wish to publish the source code of the
commercially licensed software under any other license that may be offered, a commercial
license is available for this purpose. Please review the following information to ensure
the Commercial-License requirements will be
met: https://globalobjects.de/hjacomlicense_en.html .

<b>Helmut-Jakoby-Adapted-Prosperity-Public-License</b><br>
For developers of free, non-commercial software applications who want to combine and
distribute their application with <b>Software © by Helmut Jakoby</b>, this license is
a good option. Please review the following information to ensure the
Adapted-Prosperity-Public-License requirements will be
met: https://globalobjects.de/hjappllicense_en.html .

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .

<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "EuCriticalSection.h"
//-----------------------------------------------------------------------------
namespace eut
{
  //---------------------------------------------------------------------------
  CriticalSection::CriticalSection ()
  : m_bIsLocked( false )
  {
  }
  //---------------------------------------------------------------------------
  CriticalSection::~CriticalSection()
  {
  }
  //---------------------------------------------------------------------------
  CriticalSection::CriticalSection( const CriticalSection & )
  : m_bIsLocked( false )
  {
  }
  //---------------------------------------------------------------------------
  void CriticalSection::lock()
  {
    m_ExclusiveAccessMutex.lock();
    m_bIsLocked = true;
  }
  //---------------------------------------------------------------------------
  void CriticalSection::unlock()
  {
    m_ExclusiveAccessMutex.unlock();
    m_bIsLocked = false;
  }
  //---------------------------------------------------------------------------
  bool CriticalSection::isLocked() const
  {
    return m_bIsLocked;
  }
  //---------------------------------------------------------------------------
  CriticalSection & CriticalSection::operator= ( const CriticalSection & )
  {
    m_bIsLocked = false;
    return *this;
  }
  //---------------------------------------------------------------------------
} // namespace eut
